@@ -202,6 +202,9 @@ select_row (GcalQuickAddPopover *self,
   /* Return to the events page */
   gtk_stack_set_visible_child_name (GTK_STACK (self->stack), "events");
 
+  /* Focus back the "Create" button */
+  gtk_widget_grab_focus (self->add_button);
+
   g_clear_object (&circle_pixbuf);
 }
 
@@ -257,6 +260,21 @@ update_header (GcalQuickAddPopover *self)
   g_free (dtstart);
 }
 
+static void
+update_default_calendar_row (GcalQuickAddPopover *self)
+{
+  GtkWidget *row;
+  ESource *default_source;
+
+  default_source = gcal_manager_get_default_source (self->manager);
+
+  row = get_row_for_source (self, default_source);
+  select_row (self, GTK_LIST_BOX_ROW (row));
+
+  g_clear_object (&default_source);
+}
+
+
 /*
  * Callbacks
  */
@@ -270,6 +288,10 @@ on_source_added (GcalManager         *manager,
   ESource *default_source;
   GtkWidget *row;
 
+  /* Since we can't add on read-only calendars, lets not show them at all */
+  if (!gcal_manager_is_client_writable (manager, source))
+    return;
+
   default_source = gcal_manager_get_default_source (manager);
   row = create_row_for_source (manager, source, gcal_manager_is_client_writable (manager, source));
 
@@ -292,6 +314,23 @@ on_source_changed (GcalManager         *manager,
   GdkRGBA color;
 
   row = get_row_for_source (self, source);
+
+  /* If the calendar changed from/to read-only, we add or remove it here */
+  if (!gcal_manager_is_client_writable (self->manager, source))
+    {
+      gtk_container_remove (GTK_CONTAINER (self->calendars_listbox), row);
+      return;
+    }
+  else if (!row)
+    {
+      on_source_added (self->manager,
+                       source,
+                       gcal_manager_source_enabled (self->manager, source),
+                       self);
+
+      row = get_row_for_source (self, source);
+    }
+
   color_icon = g_object_get_data (G_OBJECT (row), "color-icon");
   name_label = g_object_get_data (G_OBJECT (row), "name-label");
 
@@ -327,6 +366,9 @@ on_source_removed (GcalManager         *manager,
 
   row = get_row_for_source (self, source);
 
+  if (!row)
+    return;
+
   gtk_container_remove (GTK_CONTAINER (self->calendars_listbox), row);
 }
 
@@ -376,30 +418,32 @@ edit_or_create_event (GcalQuickAddPopover *self,
   GcalEvent *event;
   ESource *source;
   const gchar *summary;
-  gboolean all_day;
+  gboolean all_day, single_day, is_today;
 
   if (!self->selected_row)
     return;
 
+  now = g_date_time_new_now_local ();
   source = g_object_get_data (G_OBJECT (self->selected_row), "source");
 
   /*
-   * We only consider all day events when talking about multiday events.
-   * Everything else starts now and lasts 1 hour.
+   * We consider all day events multiday and/or non-today events.
+   * Events on today starts now and lasts 1 hour.
    */
-  all_day = datetime_compare_date (self->date_end, self->date_start) > 1;
+  single_day = datetime_compare_date (self->date_end, self->date_start) == 0;
+  is_today = single_day && datetime_compare_date (now, self->date_start) == 0;
+  all_day = datetime_compare_date (self->date_end, self->date_start) > 1 || !is_today;
 
   tz = all_day ? g_time_zone_new_utc () : g_time_zone_new_local ();
 
   /* Gather start date */
-  now = g_date_time_new_now_local ();
   date_start = g_date_time_new (tz,
                                 g_date_time_get_year (self->date_start),
                                 g_date_time_get_month (self->date_start),
                                 g_date_time_get_day_of_month (self->date_start),
                                 all_day ? 0 : g_date_time_get_hour (now),
                                 all_day ? 0 : g_date_time_get_minute (now),
-                                all_day ? 0 : g_date_time_get_second (now));
+                                0);
 
   /* Gather date end */
   if (self->date_end)
@@ -407,14 +451,14 @@ edit_or_create_event (GcalQuickAddPopover *self,
       date_end = g_date_time_new (tz,
                                   g_date_time_get_year (self->date_end),
                                   g_date_time_get_month (self->date_end),
-                                  g_date_time_get_day_of_month (self->date_end),
-                                  all_day ? 0 : g_date_time_get_hour (now) + 1,
+                                  g_date_time_get_day_of_month (self->date_end) + (single_day && all_day ? 1 : 0),
+                                  all_day ? 0 : g_date_time_get_hour (now) + (is_today ? 1 : 0),
                                   all_day ? 0 : g_date_time_get_minute (now),
-                                  all_day ? 0 : g_date_time_get_second (now));
+                                  0);
     }
   else
     {
-      date_end = g_date_time_add_hours (date_start, 1);
+      date_end = all_day ? g_date_time_add_days (date_start, 1) : g_date_time_add_hours (date_start, 1);
     }
 
   /* Gather the summary */
@@ -426,7 +470,7 @@ edit_or_create_event (GcalQuickAddPopover *self,
   /* Create an ECalComponent from the data above */
   component = build_component_from_details (summary, date_start, date_end);
 
-  event = gcal_event_new (source, component);
+  event = gcal_event_new (source, component, NULL);
   gcal_event_set_all_day (event, all_day);
   gcal_event_set_timezone (event, tz);
 
@@ -533,22 +577,17 @@ static void
 gcal_quick_add_popover_closed (GtkPopover *popover)
 {
   GcalQuickAddPopover *self;
-  GtkWidget *default_row;
-  ESource *default_source;
 
   self = GCAL_QUICK_ADD_POPOVER (popover);
-  default_source = gcal_manager_get_default_source (self->manager);
 
   /* Clear text */
   gtk_entry_set_text (GTK_ENTRY (self->summary_entry), "");
 
   /* Select the default row again */
-  default_row = get_row_for_source (self, default_source);
-  select_row (self, GTK_LIST_BOX_ROW (default_row));
+  update_default_calendar_row (self);
 
   g_clear_pointer (&self->date_start, g_date_time_unref);
   g_clear_pointer (&self->date_end, g_date_time_unref);
-  g_clear_object (&default_source);
 }
 
 static void
@@ -679,6 +718,7 @@ gcal_quick_add_popover_set_manager (GcalQuickAddPopover *self,
       g_signal_connect (manager, "source-added", G_CALLBACK (on_source_added), self);
       g_signal_connect (manager, "source-changed", G_CALLBACK (on_source_changed), self);
       g_signal_connect (manager, "source-removed", G_CALLBACK (on_source_removed), self);
+      g_signal_connect_swapped (manager, "notify::default-calendar", G_CALLBACK (update_default_calendar_row), self);
 
       g_object_notify (G_OBJECT (self), "manager");
     }
