@@ -725,7 +725,7 @@ show_new_event_widget (GcalView   *view,
     {
       g_clear_pointer (&window->event_creation_data->start_date, g_date_time_unref);
       g_clear_pointer (&window->event_creation_data->end_date, g_date_time_unref);
-      g_free (window->event_creation_data);
+      g_clear_pointer (&window->event_creation_data, g_free);
     }
 
   window->event_creation_data = g_new0 (NewEventData, 1);
@@ -1389,7 +1389,6 @@ gcal_window_constructed (GObject *object)
   helper_settings = g_settings_new ("org.gnome.desktop.interface");
   clock_format = g_settings_get_string (helper_settings, "clock-format");
   use_24h_format = (g_strcmp0 (clock_format, "24h") == 0);
-  g_message ("%s: %s", G_STRFUNC, clock_format);
   g_free (clock_format);
   g_object_unref (helper_settings);
 
@@ -1427,8 +1426,6 @@ gcal_window_constructed (GObject *object)
   gcal_search_view_connect (GCAL_SEARCH_VIEW (window->search_view), window->manager);
   gcal_search_view_set_time_format (GCAL_SEARCH_VIEW (window->search_view), use_24h_format);
 
-  g_message ("%s: %d", G_STRFUNC, use_24h_format);
-
   /* refresh timeout, first is fast */
   window->refresh_timeout_id = g_timeout_add (FAST_REFRESH_TIMEOUT, (GSourceFunc) refresh_sources, object);
 
@@ -1471,6 +1468,13 @@ gcal_window_finalize (GObject *object)
       g_clear_object (&window->event_to_delete);
     }
 
+  if (window->event_creation_data != NULL)
+    {
+      g_clear_pointer (&window->event_creation_data->start_date, g_date_time_unref);
+      g_clear_pointer (&window->event_creation_data->end_date, g_date_time_unref);
+      g_clear_pointer (&window->event_creation_data, g_free);
+    }
+
   g_clear_object (&window->manager);
   g_clear_object (&window->views_switcher);
 
